/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples;

import java.util.Map;

import org.ogf.graap.wsag.api.exceptions.NegotiationException;
import org.ogf.graap.wsag.server.actions.AbstractNegotiationAction;
import org.ogf.graap.wsag.server.actions.ActionInitializationException;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferType;

/**
 * Sample1NegotiateAgreementAction
 * 
 * @author Oliver Waeldrich
 * 
 */
// START SNIPPET: Sample1NegotiateAgreementAction
public class Sample1NegotiateAgreementAction extends AbstractNegotiationAction
{

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.actions.INegotiationAction#negotiate(org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferType,
     *      java.util.Map)
     */
    public NegotiationOfferType[] negotiate( NegotiationOfferType quote, Map<String, Object> context )
        throws NegotiationException
    {

        NegotiationOfferType counterOffer = NegotiationOfferType.Factory.newInstance();

        counterOffer.setName( quote.getContext().getTemplateName() );
        counterOffer.setAgreementId( quote.getAgreementId() );
        counterOffer.addNewContext().set( quote.getContext() );
        counterOffer.addNewTerms().set( quote.getTerms() );

        counterOffer.addNewNegotiationOfferContext().set( quote.getNegotiationOfferContext() );
        counterOffer.addNewNegotiationConstraints().set( quote.getNegotiationConstraints() );
        counterOffer.setOfferId( quote.getOfferId() );

        return new NegotiationOfferType[] { counterOffer };
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.actions.IAction#initialize()
     */
    @Override
    public void initialize() throws ActionInitializationException
    {
        //
        // nothing to do here
        //
    }
}
// END SNIPPET: Sample1NegotiateAgreementAction
