/* 
 * Copyright (c) 2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples;

import java.util.Map;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlObject;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionType;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementFactory;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.Negotiation;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.types.AbstractAgreementFactoryType;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.ServiceDescriptionTermType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3c.dom.Node;

/**
 * Sample implementation of an agreement factory.
 * 
 * @author Oliver Waeldrich
 * 
 * @deprecated the framework is customized by agreement actions only
 */
@Deprecated
public class SampleAgreementFactory extends AbstractAgreementFactoryType
    implements AgreementFactory
{

    private static Logger log = Logger.getLogger( SampleAgreementFactory.class );

    /**
     * default constructor
     */
    public SampleAgreementFactory()
    {
        super();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementTemplateType[] getTemplates()
    {
        // by overwriting this method you can include custom code to be executed
        // when a WSRF#GetResourceProperty request occurs
        AgreementTemplateType template1 = AgreementTemplateType.Factory.newInstance();
        template1.setTemplateId( "SampleComputeResources" );
        template1.setAgreementId( "SampleAgreementId_1" );
        template1.addNewContext();
        template1.addNewCreationConstraints();
        ServiceDescriptionTermType sdt = template1.addNewTerms().addNewAll().addNewServiceDescriptionTerm();

        // we set name and service name of the SDT
        sdt.setName( "sample_service_description_term" );
        sdt.setServiceName( "sample_service" );

        // we create the SDT content (must not be null, see schema)
        JobDefinitionType jdef = JobDefinitionType.Factory.newInstance();
        jdef.addNewJobDescription().addNewResources().addNewCandidateHosts().addHostName( "some_host" );

        Node sdtDoc = sdt.getDomNode();
        Node imported = sdtDoc.getOwnerDocument().importNode( jdef.getDomNode(), true );
        sdtDoc.appendChild( imported );

        AgreementTemplateType template2 = AgreementTemplateType.Factory.newInstance();
        template2.setTemplateId( "SampleOtherResources" );
        template2.setAgreementId( "SampleAgreementId_2" );
        template2.addNewContext();
        template2.addNewCreationConstraints();
        sdt = template2.addNewTerms().addNewAll().addNewServiceDescriptionTerm();

        // just import the JSDL document to the second SDT as well
        sdtDoc = sdt.getDomNode();
        imported = sdtDoc.getOwnerDocument().importNode( jdef.getDomNode(), true );
        sdtDoc.appendChild( imported );

        return new AgreementTemplateType[] { template1, template2 };
    }

    /**
     * {@inheritDoc}
     */
    public Agreement createAgreement( AgreementOffer offer ) throws AgreementFactoryException
    {

        SampleAgreement agreement = new SampleAgreement( offer );

        // we have to check if terms are set
        if ( ( offer.getTerms() != null ) && ( offer.getTerms().getAll() != null ) )
        {

            agreement.setTerms( offer.getTerms() );

            // now we get the SDTs
            ServiceDescriptionTermType[] sdts = offer.getTerms().getAll().getServiceDescriptionTermArray();
            if ( ( sdts != null ) && ( sdts.length == 1 ) && ( sdts[0].getName().equals( "TEST_EXCEPTION" ) ) )
            {
                String message = "No ServiceDescriptionTerms defined.";
                log.error( message );

                // if found a SDT named TEST_EXCEPTION, we fire the Exception
                throw new AgreementFactoryException( message );
            }

        }

        if ( offer.getContext() != null )
        {
            agreement.setContext( offer.getContext() );
        }

        return agreement;
    }

    /**
     * {@inheritDoc}
     */
    public Negotiation
        initiateNegotiation( NegotiationContextType context, XmlObject[] criticalExtensions,
                             XmlObject[] nonCriticalExtensions, Map<String, Object> environment )
            throws NegotiationFactoryException
    {

        /*
         * Include example for instantiating a new negotiation. This is supported by default by the
         * GenericAgreementFactory implementation.
         */
        throw new UnsupportedOperationException();
    }

}
