/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.server.actions.AbstractCreateAgreementAction;
import org.ogf.graap.wsag.server.actions.ActionInitializationException;
import org.ogf.graap.wsag.server.monitoring.MonitorableAgreement;

/**
 * SampleMonitorableAgreementAction
 * 
 * @author hrasheed
 * 
 */
public class SampleMonitoredAgreementAction extends AbstractCreateAgreementAction
{

    private static Logger log = Logger.getLogger( SampleMonitoredAgreementAction.class );

    /**
     * {@inheritDoc}
     */
    public Agreement createAgreement( AgreementOffer offer ) throws AgreementFactoryException
    {

        try
        {

            //
            // create a new instance of a domain-specific agreement implementation
            //
            SampleMonitoredAgreement sampleAgreementInstance = new SampleMonitoredAgreement( offer );

            //
            // create an instance of a monitored agreement
            //
            MonitorableAgreement monitorAgreement = new MonitorableAgreement( sampleAgreementInstance );

            //
            // set monitoring interval to 30 seconds, monitoring handler(s) are invoked once each monitoring
            // cycle
            //
            monitorAgreement.setCronExpression( "0/30 * * * * ?" );

            //
            // add a custom monitoring handler
            //
            monitorAgreement.addMonitoringHandler( new SampleServiceTermMonitor() );

            //
            // add the agreement offer to the execution context so that monitoring handler can access it later
            // on
            //
            monitorAgreement.getExecutionContext().getExecutionProperties().put( "OFFER",
                                                                                 offer.getXMLObject() );

            //
            // start the monitoring process
            //
            monitorAgreement.startMonitoring();

            //
            // return the monitored agreement
            //
            return monitorAgreement;

        }
        catch ( Exception e )
        {
            log.error( e );
            throw new AgreementFactoryException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public void initialize() throws ActionInitializationException
    {
        // nothing to do here
    }

}
