/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples.actions;

import java.util.Calendar;
import java.util.Map;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlDateTime;
import org.apache.xmlbeans.XmlDouble;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.RangeValueType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ResourcesType;
import org.ogf.graap.wsag.api.exceptions.NegotiationException;
import org.ogf.graap.wsag.server.actions.AbstractNegotiationAction;
import org.ogf.graap.wsag.server.actions.ActionInitializationException;
import org.ogf.graap.wsag4j.types.configuration.ImplementationConfigurationType;
import org.ogf.graap.wsag4j.types.scheduling.TimeConstraintDocument;
import org.ogf.graap.wsag4j.types.scheduling.TimeConstraintType;
import org.ogf.schemas.graap.wsAgreement.OfferItemType.ItemConstraint;
import org.ogf.schemas.graap.wsAgreement.ServiceDescriptionTermType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationConstraintSectionType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferItemType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferType;

/**
 * SampleNegotiateAction
 * 
 * @author hrasheed
 * 
 */
public class SampleNegotiateAction extends AbstractNegotiationAction
{

    /**
     * 
     */
    private static final int TC2_DURATION = 15;

    /**
     * 
     */
    private static final int ENDTIME2_OFFSET = 30;

    /**
     * 
     */
    private static final int TC1_DURATION = 20;

    /**
     * 
     */
    private static final int ENDTIME1_OFFSET = 20;

    private static final Logger LOG = Logger.getLogger( SampleNegotiateAction.class );

    private static final String RESOURCE_SDT_NAME = "RESOURCE_SDT";

    private static final String TIME_CONSTRAINT_SDT_NAME = "TIME_CONSTRAINT_SDT";

    private int negotiationIteration = 1;

    /**
     * {@inheritDoc}
     */
    public NegotiationOfferType[] negotiate( NegotiationOfferType offer, Map<String, Object> context )
        throws NegotiationException
    {

        try
        {

            String offerID = offer.getOfferId();

            /******************************************************************
             * Second Iteration of the Negotiation
             ******************************************************************/

            if ( negotiationIteration == 2 )
            {
                //
                // Final CounterOffer: 05 resources for 15 minutes duration with a time fame as
                // startTime = current + 5, endTime = startTime + 15
                //
                SampleNegotiationOffer counterOffer =
                    new SampleNegotiationOffer( (NegotiationOfferType) offer.copy() );

                counterOffer.setOfferId( "counterOffer_1_" + offerID );
                counterOffer.getNegotiationOfferContext().setCounterOfferTo( offerID );

                ResourcesType resources = counterOffer.getResourceDefinition();

                RangeValueType totalCountRange = RangeValueType.Factory.newInstance();
                totalCountRange.addNewExact().setDoubleValue( 5 );
                resources.setTotalResourceCount( totalCountRange );

                TimeConstraintType timeConstraint = counterOffer.getTimeConstraint();

                Calendar startTime = Calendar.getInstance();
                startTime.set( Calendar.SECOND, 0 );
                startTime.set( Calendar.MILLISECOND, 0 );
                startTime.add( Calendar.MINUTE, 5 );
                Calendar endTime = (Calendar) startTime.clone();
                endTime.add( Calendar.MINUTE, 15 );
                timeConstraint.setStartTime( startTime );
                timeConstraint.setEndTime( endTime );
                timeConstraint.setDuration( 15 );

                NegotiationConstraintSectionType constraints =
                    addCounterOfferConstraints( 1, 5, startTime, endTime );
                counterOffer.setNegotiationConstraints( constraints );

                setResourcesSDT( counterOffer, resources );
                setTimeConstraintSDT( counterOffer, timeConstraint );

                return new NegotiationOfferType[] { counterOffer.getXMLObject() };
            }

            /******************************************************************
             * First Iteration of the Negotiation
             ******************************************************************/

            //
            // CounterOffer_1: 05 resources for 20 minutes duration with a time fame as
            // startTime = current + 5, endTime = startTime + 20
            //
            SampleNegotiationOffer counterOffer1 =
                new SampleNegotiationOffer( (NegotiationOfferType) offer.copy() );

            counterOffer1.setOfferId( "counterOffer_1_" + offerID );
            counterOffer1.getNegotiationOfferContext().setCounterOfferTo( offerID );

            ResourcesType resources1 = counterOffer1.getResourceDefinition();

            RangeValueType totalCountRange1 = RangeValueType.Factory.newInstance();
            totalCountRange1.addNewExact().setDoubleValue( 5 );
            resources1.setTotalResourceCount( totalCountRange1 );

            TimeConstraintType timeConstraint1 = counterOffer1.getTimeConstraint();

            Calendar startTime1 = Calendar.getInstance();
            startTime1.set( Calendar.SECOND, 0 );
            startTime1.set( Calendar.MILLISECOND, 0 );
            startTime1.add( Calendar.MINUTE, 5 );
            Calendar endTime1 = (Calendar) startTime1.clone();
            endTime1.add( Calendar.MINUTE, ENDTIME1_OFFSET );
            timeConstraint1.setStartTime( startTime1 );
            timeConstraint1.setEndTime( endTime1 );
            timeConstraint1.setDuration( TC1_DURATION );

            //
            // negotiation constraints
            //
            NegotiationConstraintSectionType constraints1 =
                addCounterOfferConstraints( 1, 5, startTime1, endTime1 );
            counterOffer1.setNegotiationConstraints( constraints1 );

            setResourcesSDT( counterOffer1, resources1 );
            setTimeConstraintSDT( counterOffer1, timeConstraint1 );

            //
            // CounterOffer_2: 05 resources for 15 minutes duration with a time fame as
            // startTime = current + 10, endTime = startTime + 30
            //
            SampleNegotiationOffer counterOffer2 =
                new SampleNegotiationOffer( (NegotiationOfferType) offer.copy() );

            counterOffer2.setOfferId( "counterOffer_2_" + offerID );
            counterOffer2.getNegotiationOfferContext().setCounterOfferTo( offerID );

            ResourcesType resources2 = counterOffer2.getResourceDefinition();

            RangeValueType totalCountRange2 = RangeValueType.Factory.newInstance();
            totalCountRange2.addNewExact().setDoubleValue( 5 );
            resources2.setTotalResourceCount( totalCountRange2 );

            TimeConstraintType timeConstraint2 = counterOffer2.getTimeConstraint();

            Calendar startTime2 = Calendar.getInstance();
            startTime2.set( Calendar.SECOND, 0 );
            startTime2.set( Calendar.MILLISECOND, 0 );

            startTime2.add( Calendar.MINUTE, 10 );
            Calendar endTime2 = (Calendar) startTime2.clone();
            endTime2.add( Calendar.MINUTE, ENDTIME2_OFFSET );
            timeConstraint2.setStartTime( startTime2 );
            timeConstraint2.setEndTime( endTime2 );
            timeConstraint2.setDuration( TC2_DURATION );

            //
            // negotiation constraints
            //
            NegotiationConstraintSectionType constraints2 =
                addCounterOfferConstraints( 1, 5, startTime2, endTime2 );
            counterOffer2.setNegotiationConstraints( constraints2 );

            setResourcesSDT( counterOffer2, resources2 );
            setTimeConstraintSDT( counterOffer2, timeConstraint2 );

            negotiationIteration++;

            return new NegotiationOfferType[] { counterOffer1.getXMLObject(), counterOffer2.getXMLObject() };

        }
        catch ( Exception e )
        {
            LOG.error( e );
            throw new NegotiationException( e );
        }
    }

    private NegotiationConstraintSectionType addCounterOfferConstraints( double minValue, double maxValue,
                                                                         Calendar start, Calendar end )
    {

        //
        // negotiation constraints are defined based on the current availability of the resources within a
        // given time
        // frame
        //

        final String resourcesItemName =
            "ResourcesSDT_JobDefinition_JobDescription_Resources_TotalResourceCount_Exact";
        final String resourcesXPath =
            "declare namespace jsdl='http://schemas.ggf.org/jsdl/2005/11/jsdl';"
                + "declare namespace wsag='http://schemas.ggf.org/graap/2007/03/ws-agreement';"
                + "declare namespace wsag-neg='http://schemas.ogf.org/graap/2009/11/ws-agreement-negotiation';"
                + "$this/wsag:Terms/wsag:All/wsag:ServiceDescriptionTerm[@wsag:Name='RESOURCE_STD']"
                + "/jsdl:JobDefinition/jsdl:JobDescription/jsdl:Resources/jsdl:TotalResourceCount/jsdl:Exact";

        final String tcItemName = "TimeConstraintSDT_TimeConstraint";
        final String tcXPath =
            "declare namespace wsag-tc='http://schemas.wsag4j.org/2009/07/wsag4j-scheduling-extensions';"
                + "declare namespace wsag='http://schemas.ggf.org/graap/2007/03/ws-agreement';"
                + "$this/wsag:Terms/wsag:All/wsag:ServiceDescriptionTerm[@wsag:Name = 'TIME_CONSTRAINT_SDT']"
                + "/wsag4jt:TimeConstraint";

        NegotiationConstraintSectionType constraints = NegotiationConstraintSectionType.Factory.newInstance();

        // add item constraint for resources
        NegotiationOfferItemType resourcesItem = constraints.addNewItem();
        resourcesItem.setName( resourcesItemName );
        resourcesItem.setLocation( resourcesXPath );

        ItemConstraint resConstraint = resourcesItem.addNewItemConstraint();
        XmlDouble minResources = XmlDouble.Factory.newInstance();
        minResources.setDoubleValue( minValue );
        resConstraint.addNewMinInclusive().setValue( minResources );
        XmlDouble maxResources = XmlDouble.Factory.newInstance();
        maxResources.setDoubleValue( maxValue );
        resConstraint.addNewMaxInclusive().setValue( maxResources );

        // add item constraint for time frame
        NegotiationOfferItemType tcItem = constraints.addNewItem();
        tcItem.setName( tcItemName );
        tcItem.setLocation( tcXPath );

        ItemConstraint timeConstraint = tcItem.addNewItemConstraint();

        timeConstraint.addNewMinInclusive().setValue( XmlDateTime.Factory.newValue( start ) );
        timeConstraint.addNewMaxInclusive().setValue( XmlDateTime.Factory.newValue( end ) );

        return constraints;
    }

    private void setResourcesSDT( SampleNegotiationOffer negotiationOffer, ResourcesType jobResources )
        throws Exception
    {

        ServiceDescriptionTermType resourcesSDT = null;

        ServiceDescriptionTermType[] sdts =
            negotiationOffer.getTerms().getAll().getServiceDescriptionTermArray();

        if ( sdts != null )
        {
            for ( int i = 0; i < sdts.length; i++ )
            {
                if ( sdts[i].getName().equals( RESOURCE_SDT_NAME ) )
                {
                    resourcesSDT = sdts[i];
                    break;
                }
            }
        }

        String name = resourcesSDT.getName();
        String serviceName = resourcesSDT.getServiceName();

        JobDefinitionDocument resourcesDoc = JobDefinitionDocument.Factory.newInstance();
        resourcesDoc.addNewJobDefinition().addNewJobDescription().addNewResources();
        resourcesDoc.getJobDefinition().getJobDescription().getResources().set( jobResources );

        resourcesSDT.set( resourcesDoc );
        resourcesSDT.setName( name );
        resourcesSDT.setServiceName( serviceName );
    }

    private void setTimeConstraintSDT( SampleNegotiationOffer negotiationOffer,
                                       TimeConstraintType timeConstraint )
    {

        ServiceDescriptionTermType timeConstraintSDT = null;

        ServiceDescriptionTermType[] sdts =
            negotiationOffer.getTerms().getAll().getServiceDescriptionTermArray();

        if ( sdts != null )
        {
            for ( int i = 0; i < sdts.length; i++ )
            {
                if ( sdts[i].getName().equals( TIME_CONSTRAINT_SDT_NAME ) )
                {
                    timeConstraintSDT = sdts[i];
                    break;
                }
            }
        }

        String name = timeConstraintSDT.getName();
        String serviceName = timeConstraintSDT.getServiceName();

        TimeConstraintDocument timeConstraintDoc = TimeConstraintDocument.Factory.newInstance();
        timeConstraintDoc.addNewTimeConstraint();
        timeConstraintDoc.getTimeConstraint().set( timeConstraint );

        timeConstraintSDT.set( timeConstraintDoc );
        timeConstraintSDT.setName( name );
        timeConstraintSDT.setServiceName( serviceName );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void initialize() throws ActionInitializationException
    {
        //
        // You can access the action configuration data of the wsag4j-engine.config
        // by calling the getConfiguration() method. You can iterate over the configuration
        // data by using the DOM API, or you can select child data via XPath.
        //
        ImplementationConfigurationType config = getHandlerContext().getHandlerConfiguration();
        config.getDomNode();
        //
        // put additional code here to initialize your action
        //
    }
}
