/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples.pendingagreement;

import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.server.actions.AbstractCreateAgreementAction;
import org.ogf.graap.wsag.server.actions.ActionInitializationException;

/**
 * CreatePendingAgreementAction
 * 
 * This class shows an example for a long lasting agreement creation process.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class CreatePendingAgreementAction extends AbstractCreateAgreementAction
{

    /**
     * 
     */
    private static final int DEFAULT_WAITING_TIME = 5000;

    /**
     * Example for a long lasting agreement creation process.
     * 
     * {@inheritDoc}
     */
    public Agreement createAgreement( AgreementOffer offer ) throws AgreementFactoryException
    {
        //
        // This is a CreateAgreementAction that takes some time.
        // Therefore, we go to sleep for a while...
        //
        try
        {
            Thread.sleep( DEFAULT_WAITING_TIME );
        }
        catch ( InterruptedException e )
        {
            // do nothing
        }

        //
        // Now, since the work is done and we return the agreement...
        //
        return new PendingAgreement( offer );
    }

    /**
     * Initializes the action.
     * 
     * {@inheritDoc}
     */
    public void initialize() throws ActionInitializationException
    {
        // nothing to do here
    }

}
