/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples.site;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.types.AbstractAgreementType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputType;
import org.w3c.dom.Node;

/**
 * This agreement implementation is used as example for the project documentation.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class ExampleAgreement extends AbstractAgreementType
{

    private static Logger log = Logger.getLogger( ExampleAgreement.class );

    private ExampleService service;

    /**
     * Creates an new agreement instance based on an agreement offer. The agreement properties of the
     * agreement are initialized based on the offer. Additionally, for each service description term
     * (guarantee term) specified in the offer a corresponding service term state (guarantee term state) is
     * initialized.
     * 
     * @param offer
     *            the agreement offer for which the agreement is created
     */
    public ExampleAgreement( AgreementOffer offer )
    {
        super( offer );
    }

    /**
     * Sets the service associated with this agreement instance.
     * 
     * @param service
     *            the service to set
     */
    public void setService( ExampleService service )
    {
        this.service = service;
    }

    /**
     * This method is invoked when an agreement is terminated. Implementations can add domain-specific
     * functionality here in order to shutdown services or free resources.
     * 
     * {@inheritDoc}
     */
    public void terminate( TerminateInputType reason )
    {
        Node terminateReson = reason.getDomNode().getFirstChild();
        if ( terminateReson != null )
        {
            // find out what the reason is and take the appropriate action
            if ( log.isInfoEnabled() )
            {
                log.info( "The greement was terminated for the following reason: " + reason.xmlText() );
            }
        }

        //
        // Implementations can add domain-specific code here, for example
        // to terminate associated services and free allocated resources.
        //
        service.interrupt();
    }

}
